import requests
from urllib.parse import urlencode, parse_qsl
from datetime import datetime, timedelta
from pytz import UTC
from dateutil.parser import parse as parse_datetime
from sqlalchemy.exc import InternalError, IntegrityError, DataError
from sqlalchemy import and_, or_, inspect, alias, text, func, select, case, cast
from sqlalchemy.orm import aliased
from api_dnl import model
from api_dnl.utils.statisticapi2 import cdr_fields

import falcon
from falcon_rest import conf
from falcon_rest.logger import log
from falcon_rest import schemes
from falcon_rest.responses import responses
from falcon_rest.responses import errors
from falcon_rest.resources.resources import swagger, ATTRIBUTE_ERROR_RE
from falcon_rest.helpers import check_permission
from api_dnl.model import ImportExportLogs

from api_dnl.resources import check_context, DnlList, ValidationError, DnlCreate, DnlResource, CustomGetAction, \
    CustomPostAction, \
    CustomDeleteAction, CustomPatchAction
from api_dnl.schemes.cloud import *
from api_dnl.view import DEFAULT_SECURITY, OperationErrorResponse
import json


def db_man_base():
    conf = model.DnlCloudDbmanCfg.filter(model.DnlCloudDbmanCfg.server_ip == '127.0.0.1').first()
    if not conf:
        raise Exception("Cloud Database Manager not configured. Call a tech personnel")
    return '{}://{}:{}'.format('https' if conf.server_use_ssl else 'http', conf.server_ip, conf.server_port)


def fd_base():
    conf = model.DnlCloudDownloaderCfg.filter(model.DnlCloudDownloaderCfg.server_ip == '127.0.0.1').first()
    if not conf:
        raise Exception("Cloud Downloader not configured. Call a tech personnel")
    return '{}://{}:{}'.format('https' if conf.server_use_ssl else 'http', conf.server_ip, conf.server_port)


def ds_base():
    conf = model.DnlCloudSearchCfg.filter(model.DnlCloudSearchCfg.server_ip == '127.0.0.1').first()
    if not conf:
        conf = model.DnlCloudSearchCfg.filter().first()
        if not conf:
            raise Exception("Cloud Search not configured. Call a tech personnel")
    return '{}://{}:{}'.format('https' if conf.server_use_ssl else 'http', conf.server_ip, conf.server_port)


# region +++DnlCloudDbmanCfg+++
class DnlCloudDbmanCfgCreate(DnlCreate):
    scheme_class = DnlCloudDbmanCfgScheme
    model_class = model.DnlCloudDbmanCfg
    entity = 'DnlCloudDbmanCfg'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        return obj


class DnlCloudDbmanCfgResource(DnlResource):
    model_class = model.DnlCloudDbmanCfg
    scheme_class = DnlCloudDbmanCfgScheme
    scheme_class_get = DnlCloudDbmanCfgSchemeGet
    scheme_class_modify = DnlCloudDbmanCfgSchemeModify
    entity = 'DnlCloudDbmanCfg'
    id_field = 'server_name'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()


class DnlCloudDbmanCfgList(DnlList):
    scheme_class = DnlCloudDbmanCfgSchemeGet
    model_class = model.DnlCloudDbmanCfg
    entity_plural = 'DnlCloudDbmanCfgs'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudDbmanCfg---
# region +++DnlCloudDownloaderCfg+++
class DnlCloudDownloaderCfgCreate(DnlCreate):
    scheme_class = DnlCloudDownloaderCfgScheme
    model_class = model.DnlCloudDownloaderCfg
    entity = 'DnlCloudDownloaderCfg'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        return obj


class DnlCloudDownloaderCfgResource(DnlResource):
    model_class = model.DnlCloudDownloaderCfg
    scheme_class = DnlCloudDownloaderCfgScheme
    scheme_class_get = DnlCloudDownloaderCfgSchemeGet
    scheme_class_modify = DnlCloudDownloaderCfgSchemeModify
    entity = 'DnlCloudDownloaderCfg'
    id_field = 'server_name'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()


class DnlCloudDownloaderCfgList(DnlList):
    scheme_class = DnlCloudDownloaderCfgSchemeGet
    model_class = model.DnlCloudDownloaderCfg
    entity_plural = 'DnlCloudDownloaderCfgs'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudDownloaderCfg---
# region +++DnlCloudSearchCfg+++
class DnlCloudSearchCfgCreate(DnlCreate):
    scheme_class = DnlCloudSearchCfgScheme
    model_class = model.DnlCloudSearchCfg
    entity = 'DnlCloudSearchCfg'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        return obj


class DnlCloudSearchCfgResource(DnlResource):
    model_class = model.DnlCloudSearchCfg
    scheme_class = DnlCloudSearchCfgScheme
    scheme_class_get = DnlCloudSearchCfgSchemeGet
    scheme_class_modify = DnlCloudSearchCfgSchemeModify
    entity = 'DnlCloudSearchCfg'
    id_field = 'server_name'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()


class DnlCloudSearchCfgList(DnlList):
    scheme_class = DnlCloudSearchCfgSchemeGet
    model_class = model.DnlCloudSearchCfg
    entity_plural = 'DnlCloudSearchCfgs'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudSearchCfg---+
# region +++DnlCloudLog+++

class DnlCloudLogList(DnlList):
    scheme_class = DnlCloudLogSchemeGet
    model_class = model.DnlCloudLog
    entity_plural = 'DnlCloudLogs'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudLog---
# region +++DnlCloudStorage+++
class DnlCloudStorageCreate(DnlCreate):
    scheme_class = DnlCloudStorageScheme
    model_class = model.DnlCloudStorage
    entity = 'DnlCloudStorage'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):

        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        if obj.type == 'ftp' and obj.ftp:
            if obj.ftp.netrc_id:
                f = ImportExportLogs.get(obj.ftp.netrc_id)
                obj.ftp.netrc_path = f.file_path + '/' + f.myfile_filename
                del obj.ftp.netrc_id
        if obj.type == 'sftp' and obj.sftp:
            if obj.sftp.pubkey_id:
                f = ImportExportLogs.get(obj.sftp.pubkey_id)
                obj.sftp.pubkey_path = f.file_path + '/' + f.myfile_filename
                del obj.sftp.pubkey_id
            if obj.sftp.privkey_id:
                f = ImportExportLogs.get(obj.sftp.privkey_id)
                obj.sftp.priv_path = f.file_path + '/' + f.myfile_filename
                del obj.sftp.privkey_id
        if obj.type == 'gcloud' and obj.gcloud:
            if obj.type == 'oauth2':
                if not obj.email or not obj.secret or not obj.token:
                    raise ValidationError('When type=oauth2 then email,secret,token are mandatory!')
            if obj.type == 'service':
                if not obj.key_file_id:
                    raise ValidationError('When type=service then json key_file should be uploaded!')
            if obj.gcloud.key_file_id:
                f = ImportExportLogs.get(obj.gcloud.key_file_id)
                obj.gcloud.key_file_path = f.file_path + '/' + f.myfile_filename
                del obj.gcloud.key_file_id

        return obj


class DnlCloudStorageResource(DnlResource):
    model_class = model.DnlCloudStorage
    scheme_class = DnlCloudStorageScheme
    scheme_class_get = DnlCloudStorageSchemeGet
    scheme_class_modify = DnlCloudStorageSchemeModify
    entity = 'DnlCloudStorage'
    id_field = 'uuid'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()

    def before_update(self, obj, req):
        if obj.ftp and 'ftp' in req.data:
            req.data['ftp']['uuid'] = obj.uuid
        if obj.sftp and 'sftp' in req.data:
            req.data['sftp']['uuid'] = obj.uuid
            if obj.sftp.pubkey_id:
                f = ImportExportLogs.get(obj.sftp.pubkey_id)
                obj.sftp.pubkey_path = f.file_path + '/' + f.myfile_filename
                del obj.sftp.pubkey_id
            if obj.sftp.privkey_id:
                f = ImportExportLogs.get(obj.sftp.privkey_id)
                obj.sftp.priv_path = f.file_path + '/' + f.myfile_filename
                del obj.sftp.privkey_id
        if obj.gcloud and 'gcloud' in req.data:
            req.data['gcloud']['uuid'] = obj.uuid
            ob = req.data['gcloud']

            class _R():
                pass

            obj2 = _R()
            obj2.__dict__ = ob
            # if obj2.type == 'oauth2':
            #     if not getattr(obj2, 'email', None) or not getattr(obj2, 'secret', None) or not getattr(obj2, 'token', None):
            #         raise ValidationError('When type=oauth2 then email,secret,token are mandatory!')
            if obj2.type == 'service':
                if not getattr(obj2, 'key_file_id', None):
                    raise ValidationError('When type=service then json key_file should be uploaded!')

            if obj.gcloud.key_file_id:
                f = ImportExportLogs.get(obj.gcloud.key_file_id)
                obj.gcloud.key_file_path = f.file_path + '/' + f.myfile_filename
                del obj.gcloud.key_file_id
        return obj


class DnlCloudStorageList(DnlList):
    scheme_class = DnlCloudStorageSchemeGet
    model_class = model.DnlCloudStorage
    entity_plural = 'DnlCloudStorages'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudStorage---
# region +++DnlCloudStatus+++
class DnlCloudStatusCreate(DnlCreate):
    scheme_class = DnlCloudStatusScheme
    model_class = model.DnlCloudStatus
    entity = 'DnlCloudStatus'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def before_create(self, obj, **kwargs):
        user = self.get_user(self.req)
        # obj.created_by=user.name
        # obj.created_on=datetime.now(UTC)
        return obj


class DnlCloudStatusResource(DnlResource):
    model_class = model.DnlCloudStatus
    scheme_class = DnlCloudStatusScheme
    scheme_class_get = DnlCloudStatusSchemeGet
    scheme_class_modify = DnlCloudStatusSchemeModify
    entity = 'DnlCloudStatus'
    id_field = 'server_name'
    security = (DEFAULT_SECURITY)
    path_parameters = ()
    restrict = ()


class DnlCloudStatusList(DnlList):
    scheme_class = DnlCloudStatusSchemeGet
    model_class = model.DnlCloudStatus
    entity_plural = 'DnlCloudStatuss'
    path_parameters = ()
    security = (DEFAULT_SECURITY)
    restrict = ()

    def modify_query_from_filtering_for_list(self, filtering, **kwargs):
        filt, ret = super().modify_query_from_filtering_for_list(filtering, **kwargs)
        user = self.get_user(self.req)
        if not user.is_admin:
            cls = self.model_class
            # ret = ret.filter(cls.pool_id != 0)#TODO:filter for user
        return filt, ret


# endregion ---DnlCloudStatus---

# region DB manager

class CloudGet(CustomGetAction):
    description = "Custom get"
    path_parameters = ()
    allow_methods = ["get"]
    cloud_method = '/{5}/switch/list'

    @property
    def cloud_base(self):
        return db_man_base()

    @property
    def cloud_url(self):
        if '{' in self.cloud_method:
            return '{}{}'.format(self.cloud_base, self.cloud_method.format(*self.req.path.split('/')))
        return '{}{}'.format(self.cloud_base, self.cloud_method)

    def on_get(self, req, resp, **kwargs):
        self.init_req(req)
        if not check_context(self, req, resp, **kwargs):
            return False
        self.req_uuid = None
        if 'req_uuid' in kwargs:
            self.req_uuid = kwargs['req_uuid']
        user = self.get_user(self.req)
        # params = req.data
        params = dict(parse_qsl(req.query_string))
        # cloud_base = 'http://localhost:33500'
        # url = '{}/switch/list'.format(cloud_base)
        try:
            url = self.cloud_url + '?' + req.query_string if req.query_string else self.cloud_url
            log.debug('CLOUD GET url: {} params:{}'.format(url, params))
            ret = requests.get(url, verify=False)
            log.debug('CLOUD GET status:{} response:{}'.format(ret.status_code, resp.body))
            # if ret.status_code in (200, 201, 204):
            #     resp.content_type = 'application/json'
            #     resp.body = ret.content
            #     resp.status = falcon.HTTP_200
            if 'Content-Type' in ret.headers:
                resp.content_type = ret.headers['Content-type']
            if 'Content-Disposition' in ret.headers:
                resp.set_header('Content-Disposition', ret.headers['Content-Disposition'])
            if 'Content-Length' in ret.headers:
                resp.set_header('Content-Length', ret.headers['Content-Length'])
            if 'Date' in ret.headers:
                resp.set_header('Date', ret.headers['Date'])
            resp.data = ret.content
            resp.status = str(ret.status_code)
        except Exception as e:
            self.set_response(resp, OperationErrorResponse(e))
            return None
        pass


class CloudSwitchListGet(CloudGet):
    description = "Switch list"
    cloud_method = '/{5}/switch/list'


class CloudStatusGet(CloudGet):
    description = "Status"
    cloud_method = '/{5}/status'


class CloudPost(CustomPostAction):
    description = "Custom post"

    @property
    def cloud_base(self):
        return db_man_base()

    @property
    def cloud_url(self):
        if '{' in self.cloud_method:
            return '{}{}'.format(self.cloud_base, self.cloud_method.format_map(self.req.patch_parameters))
        return '{}{}'.format(self.cloud_base, self.cloud_method)

    def on_post(self, req, resp, **kwargs):
        self.init_req(req)
        if not check_context(self, req, resp, **kwargs):
            return False
        user = self.get_user(self.req)
        log.debug('user:{}'.format(user))
        params = req.data

        try:
            log.debug('CLOUD POST url: {} params:{}'.format(self.cloud_url, params))
            ret = requests.post(self.cloud_url, data=params, verify=False)
            log.debug('CLOUD POST status:{} response:{}'.format(ret.status_code, resp.body))
            # if ret:
            resp.content_type = 'application/json'
            resp.body = ret.content
            resp.status = str(ret.status_code)  # falcon.HTTP_200
        except Exception as e:
            self.set_response(resp, OperationErrorResponse(e))
            return None
        pass


class CloudPost(CloudPost):
    description = "Reload configuration"
    cloud_method = '/{5}/reload'

    # path_parameters = ({"name": "request_id", "type": "string", "description": "request id"},)
    @property
    def cloud_base(self):
        return db_man_base()


class CloudRestart(CloudPost):
    description = "Restart daemons"
    cloud_method = '/{5}/restart'


class CloudShutdown(CloudPost):
    description = "Shutdown daemons"
    cloud_method = '/{5}/shutdown'


class CloudReindex(CloudPost):
    description = "Reindex db"
    cloud_method = '/{5}/reindex'


class CloudRunBackup(CloudPost):
    description = "RunBackup"
    cloud_method = '/{5}/run-backup'


class CloudSwitchAdd(CloudPost):
    body_parameters = ('Switch params', CloudSwitchScheme,)
    description = "Add switch"
    cloud_method = '/{5}/switch/add'


class CloudSwitchDelete(CloudPost):
    body_parameters = ('Asynchronous pcap files search', CloudSwitchDeleteScheme,)
    description = "Delete switch"
    cloud_method = '/{5}/switch/delete'


# endregion
# region File downloader
class CloudServerFileAction(CustomPostAction):
    description = "File  action"
    cloud_method = '/{6}/{7}'

    @property
    def cloud_base(self):
        return ds_base()


class CloudServerFileStatus(CloudGet):
    description = "Cdr files list"
    cloud_method = '/{6}/status'

    @property
    def cloud_base(self):
        return fd_base()


class CloudFileListCdr(CloudGet):
    description = "Cdr files list"
    cloud_method = '/list/cdr'
    query_parameters = ({"name": "switch_name", "type": "string", 'required': True, "description": "name of a switch"},)

    @property
    def cloud_base(self):
        return fd_base()
    #     conf = model.DnlCloudDownloaderCfg.filter(model.DnlCloudDownloaderCfg.server_ip == '127.0.0.1').first()
    #     return fd_base() +'{}'.format(conf.server_name)


class CloudFileListPcap(CloudFileListCdr):
    description = "pcap files list"
    cloud_method = '/list/pcap'
    query_parameters = ({"name": "switch_name", "type": "string", 'required': True, "description": "name of a switch"},)


class CloudServerFileListCdr(CloudGet):
    description = "Cdr files list"
    cloud_method = '/{6}/list/cdr'
    path_parameters = ({"name": "server_name", "type": "string", "description": "name of an instance"},)

    @property
    def cloud_base(self):
        return fd_base()


class CloudServerFileListPcap(CloudFileListCdr):
    description = "pcap files list"
    cloud_method = '/{6}/list/pcap'
    path_parameters = ({"name": "server_name", "type": "string", "description": "name of an instance"},)


class CloudFileGetCdr(CloudFileListCdr):
    description = "cdr file"
    cloud_method = '/get/cdr'
    query_parameters = ({"name": "switch_name", "type": "string", 'required': True, "description": "name of a switch"},
                        {"name": "timestamp", "type": "integer", 'required': True, "description": "timestamp of file"},
                        )


class CloudFileGetPcap(CloudFileListCdr):
    description = "pcap file"
    cloud_method = '/get/pcap'
    query_parameters = ({"name": "switch_name", "type": "string", 'required': True, "description": "name of a switch"},
                        {"name": "timestamp", "type": "integer", 'required': True, "description": "timestamp of file"},
                        )


# endregion File downloader
# region Database search

# endregion Database search


class CloudSearchCdr(CloudPost):
    body_parameters = ('Asynchronous cdr files search', CloudCdrScheme,)
    description = "cdr"
    cloud_method = '/cdr'

    @property
    def cloud_base(self):
        return ds_base()

    def on_post(self, req, resp, **kwargs):
        if 'tz' in req.data:
            tz = int(req.data.pop('tz'))
            req.data['start_time'] += tz * 86400
            req.data['end_time'] += tz * 86400
        if 'ingress_carrier_id' in req.data:
            ingress_carrier_id = int(req.data.pop('ingress_carrier_id'))
            resources = model.Resource.filter(and_(model.Resource.client_id == ingress_carrier_id, model.Resource.ingress))
            ingress_trunk_ids = [str(i.resource_id) for i in resources]
            ingress_trunk_id = req.data.pop('ingress_trunk_id', '').split(',')
            if ingress_trunk_id != ['']:
                ingress_trunk_ids = list(set(ingress_trunk_ids).intersection(set(ingress_trunk_id)))
            req.data['ingress_trunk_id'] = ','.join(ingress_trunk_ids)
        if 'egress_carrier_id' in req.data:
            egress_carrier_id = int(req.data.pop('egress_carrier_id'))
            resources = model.Resource.filter(and_(model.Resource.client_id == egress_carrier_id, model.Resource.egress))
            egress_trunk_ids = [str(i.resource_id) for i in resources]
            egress_trunk_id = req.data.pop('egress_trunk_id', '').split(',')
            if egress_trunk_id != ['']:
                egress_trunk_ids = list(set(egress_trunk_ids).intersection(set(egress_trunk_id)))
            req.data['egress_trunk_id'] = ','.join(egress_trunk_ids)
        if 'fields' in req.data:
            fields = req.data['fields'].split(',')
            int_fields = []
            for field in fields:
                cdr_field = model.DailyCdrField.filter(model.DailyCdrField.field == field).first()
                if cdr_field and cdr_field.id <= 107:
                    int_fields += [str(cdr_field.id)]
            req.data['fields'] = ','.join(int_fields)
        return super().on_post(req, resp, **kwargs)


class CloudSearchPcap(CloudPost):
    body_parameters = ('Asynchronous pcap files search', CloudPcapScheme,)
    description = "pcap"
    cloud_method = '/pcap'

    @property
    def cloud_base(self):
        return ds_base()


class CloudSearchGet(CloudGet):
    description = "get search query results"
    cloud_method = ''
    path_parameters = ({"name": "req_uuid", "type": "string", "description": "request id"},)

    def on_post(self, req, resp, **kwargs):
        if 'tz' in req.data:
            tz = int(req.data.pop('tz'))
            req.data['start_time'] += tz * 86400
            req.data['end_time'] += tz * 86400
        if 'ingress_carrier_id' in req.data:
            ingress_carrier_id = int(req.data.pop('ingress_carrier_id'))
            resources = model.Resource.filter(and_(model.Resource.client_id == ingress_carrier_id, model.Resource.ingress))
            ingress_trunk_ids = [str(i.resource_id) for i in resources]
            ingress_trunk_id = req.data.pop('ingress_trunk_id', '').split(',')
            if ingress_trunk_id:
                ingress_trunk_ids = list(set(ingress_trunk_ids).intersection(set(ingress_trunk_id)))
            req.data['ingress_trunk_id'] = ','.join(ingress_trunk_ids)
        if 'egress_carrier_id' in req.data:
            egress_carrier_id = int(req.data.pop('egress_carrier_id'))
            resources = model.Resource.filter(and_(model.Resource.client_id == egress_carrier_id, model.Resource.egress))
            egress_trunk_ids = [str(i.resource_id) for i in resources]
            egress_trunk_id = req.data.pop('egress_trunk_id', '').split(',')
            if egress_trunk_id:
                egress_trunk_ids = list(set(egress_trunk_ids).intersection(set(egress_trunk_id)))
            req.data['egress_trunk_id'] = ','.join(egress_trunk_ids)
        return super().on_post(req, resp, **kwargs)

    @property
    def cloud_base(self):
        return ds_base()

    @property
    def cloud_url(self):
        return '{}/{}{}'.format(self.cloud_base, self.req_uuid, self.cloud_method)


class CloudSearchGetList(CloudGet):
    description = "get search all cloud query requests"
    cloud_method = '/list'
    query_parameters = ({"name": "start_time", "type": "integer", "description": "time filter"},
                        {"name": "end_time", "type": "integer", "description": "time filter"},
                        )

    @property
    def cloud_base(self):
        return ds_base()


class CloudSearchGetListCdr(CloudGet):
    description = "get search CDR query requests"
    cloud_method = '/list/cdr'
    query_parameters = ({"name": "start_time", "type": "integer", "description": "time filter"},
                        {"name": "end_time", "type": "integer", "description": "time filter"},
                        )

    @property
    def cloud_base(self):
        return ds_base()


class CloudSearchGetListPcap(CloudGet):
    description = "get search PCAP query requests"
    cloud_method = '/list/pcap'
    query_parameters = ({"name": "start_time", "type": "integer", "description": "time filter"},
                        {"name": "end_time", "type": "integer", "description": "time filter"},
                        )

    @property
    def cloud_base(self):
        return ds_base()


class CloudSearchDownload(CloudSearchGet):
    description = "get search query results"
    cloud_method = '/download'

    def on_get(self, req, resp, **kwargs):
        super().on_get(req, resp, **kwargs)
        try:
            resp.data = resp.data.replace(b'?', b',')
        except Exception as e:
            log.debug('cloud search download error {}'.format(str(e)))


class CloudReqDelete(CustomDeleteAction):
    description = "Switch list"
    path_parameters = ({"name": "req_uuid", "type": "string", "description": "request id"},)
    cloud_method = ''

    @property
    def cloud_base(self):
        return ds_base()

    @property
    def cloud_url(self):
        return '{}/{}{}'.format(self.cloud_base, self.req_uuid, self.cloud_method)

    def on_delete(self, req, resp, **kwargs):
        self.init_req(req)
        if not check_context(self, req, resp, **kwargs):
            return False
        self.req_uuid = None
        if 'req_uuid' in kwargs:
            self.req_uuid = kwargs['req_uuid']
        user = self.get_user(self.req)
        # params = req.data
        params = dict(parse_qsl(req.query_string))
        try:
            url = self.cloud_url + '?' + req.query_string if req.query_string else self.cloud_url
            ret = requests.delete(url)
            resp.body = ret.content
            resp.status = str(ret.status_code)
        except Exception as e:
            self.set_response(resp, OperationErrorResponse(e))
            return None
        pass


class CloudServerSearchAction(CustomPostAction):
    description = "File  action"
    cloud_method = '/{6}/{7}'

    @property
    def cloud_base(self):
        return ds_base()


class CloudServerSearchStatus(CloudGet):
    description = "Cdr files list"
    cloud_method = '/{6}/status'

    @property
    def cloud_base(self):
        return ds_base()
